@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+/*  GMime
+ *  Copyright (C) 2000-2011 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ static const char *path = "/usr/bin/gpg";
 static const char *passphrase = "no.secret";
 
 static gboolean
-request_passwd (GMimeCipherContext *ctx, const char *user_id, const char *prompt_ctx, gboolean reprompt, GMimeStream *response, GError **err)
+request_passwd (GMimeCryptoContext *ctx, const char *user_id, const char *prompt_ctx, gboolean reprompt, GMimeStream *response, GError **err)
 {
 	if (g_mime_stream_write_string (response, passphrase) == -1 ||
 	    g_mime_stream_write (response, "\n", 1) == -1) {
@@ -136,16 +136,32 @@ count_parts_in_message (GMimeMessage *message)
 }
 
 #ifndef G_OS_WIN32
+static GMimeSignerStatus
+sig_status (GMimeSignatureValidity *validity)
+{
+	GMimeSignerStatus status = GMIME_SIGNER_STATUS_GOOD;
+	GMimeSigner *signer = validity->signers;
+	
+	if (signer == NULL)
+		return GMIME_SIGNER_STATUS_ERROR;
+	
+	while (signer != NULL) {
+		status = MAX (status, signer->status);
+		signer = signer->next;
+	}
+	
+	return status;
+}
+
 static void
 verify_foreach_callback (GMimeObject *parent, GMimeObject *part, gpointer user_data)
 {
-	GMimeCipherContext *ctx = user_data;
+	GMimeCryptoContext *ctx = user_data;
 	
 	if (GMIME_IS_MULTIPART_SIGNED (part)) {
 		/* this is a multipart/signed part, so we can verify the pgp signature */
 		GMimeMultipartSigned *mps = (GMimeMultipartSigned *) part;
 		GMimeSignatureValidity *validity;
-		GMimeSignatureStatus status;
 		GError *err = NULL;
 		const char *str;
 		
@@ -160,16 +176,15 @@ verify_foreach_callback (GMimeObject *parent, GMimeObject *part, gpointer user_d
 			g_error_free (err);
 		} else {
 			/* print out validity info - GOOD vs BAD and "why" */
-			status = g_mime_signature_validity_get_status (validity);
-			switch (status) {
-			case GMIME_SIGNATURE_STATUS_GOOD:
+			switch (sig_status (validity)) {
+			case GMIME_SIGNER_STATUS_GOOD:
 				str = "Good";
 				break;
-			case GMIME_SIGNATURE_STATUS_BAD:
+			case GMIME_SIGNER_STATUS_BAD:
 				str = "Bad";
 				break;
-			case GMIME_SIGNATURE_STATUS_UNKNOWN:
-				str = "Unknown";
+			case GMIME_SIGNER_STATUS_ERROR:
+				str = "Error";
 				break;
 			default:
 				str = NULL;
@@ -185,7 +200,7 @@ verify_foreach_callback (GMimeObject *parent, GMimeObject *part, gpointer user_d
 }
 
 static void
-verify_signed_parts (GMimeMessage *message, GMimeCipherContext *ctx)
+verify_signed_parts (GMimeMessage *message, GMimeCryptoContext *ctx)
 {
 	/* descend the mime tree and verify any signed parts */
 	g_mime_message_foreach (message, verify_foreach_callback, ctx);
@@ -283,7 +298,7 @@ remove_a_mime_part (GMimeMessage *message)
 int main (int argc, char **argv)
 {
 #ifndef G_OS_WIN32
-	GMimeCipherContext *ctx;
+	GMimeCryptoContext *ctx;
 #endif
 	GMimeMessage *message;
 	int fd;
@@ -309,7 +324,7 @@ int main (int argc, char **argv)
 	
 #ifndef G_OS_WIN32
 #ifdef ENABLE_CRYPTOGRAPHY
-	/* create our cipher context */
+	/* create our crypto context */
 	ctx = g_mime_gpg_context_new (request_passwd, path);
 	
 	/* don't allow auto key-retrival */
