@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2011 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -46,6 +46,12 @@ static GMimeCryptoHash crypto_hash_id (GMimeCryptoContext *ctx, const char *hash
 
 static const char *crypto_hash_name (GMimeCryptoContext *ctx, GMimeCryptoHash hash);
 
+static const char *crypto_get_signature_protocol (GMimeCryptoContext *ctx);
+
+static const char *crypto_get_encryption_protocol (GMimeCryptoContext *ctx);
+
+static const char *crypto_get_key_exchange_protocol (GMimeCryptoContext *ctx);
+
 static int crypto_sign (GMimeCryptoContext *ctx, const char *userid,
 			GMimeCryptoHash hash, GMimeStream *istream,
 			GMimeStream *ostream, GError **err);
@@ -59,8 +65,8 @@ static int crypto_encrypt (GMimeCryptoContext *ctx, gboolean sign,
 			   GPtrArray *recipients, GMimeStream *istream,
 			   GMimeStream *ostream, GError **err);
 
-static GMimeSignatureValidity *crypto_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
-					       GMimeStream *ostream, GError **err);
+static GMimeDecryptionResult *crypto_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
+					      GMimeStream *ostream, GError **err);
 
 static int crypto_import_keys (GMimeCryptoContext *ctx, GMimeStream *istream,
 			       GError **err);
@@ -114,6 +120,9 @@ g_mime_crypto_context_class_init (GMimeCryptoContextClass *klass)
 	klass->decrypt = crypto_decrypt;
 	klass->import_keys = crypto_import_keys;
 	klass->export_keys = crypto_export_keys;
+	klass->get_signature_protocol = crypto_get_signature_protocol;
+	klass->get_encryption_protocol = crypto_get_encryption_protocol;
+	klass->get_key_exchange_protocol = crypto_get_key_exchange_protocol;
 }
 
 static void
@@ -217,6 +226,78 @@ g_mime_crypto_context_hash_name (GMimeCryptoContext *ctx, GMimeCryptoHash hash)
 }
 
 
+static const char *
+crypto_get_signature_protocol (GMimeCryptoContext *ctx)
+{
+	return NULL;
+}
+
+
+/**
+ * g_mime_crypto_context_get_signature_protocol:
+ * @ctx: a #GMimeCryptoContext
+ *
+ * Gets the signature protocol for the crypto context.
+ *
+ * Returns: the signature protocol or %NULL if not supported.
+ **/
+const char *
+g_mime_crypto_context_get_signature_protocol (GMimeCryptoContext *ctx)
+{
+	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
+	
+	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->get_signature_protocol (ctx);
+}
+
+
+static const char *
+crypto_get_encryption_protocol (GMimeCryptoContext *ctx)
+{
+	return NULL;
+}
+
+
+/**
+ * g_mime_crypto_context_get_encryption_protocol:
+ * @ctx: a #GMimeCryptoContext
+ *
+ * Gets the encryption protocol for the crypto context.
+ *
+ * Returns: the encryption protocol or %NULL if not supported.
+ **/
+const char *
+g_mime_crypto_context_get_encryption_protocol (GMimeCryptoContext *ctx)
+{
+	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
+	
+	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->get_encryption_protocol (ctx);
+}
+
+
+static const char *
+crypto_get_key_exchange_protocol (GMimeCryptoContext *ctx)
+{
+	return NULL;
+}
+
+
+/**
+ * g_mime_crypto_context_get_key_exchange_protocol:
+ * @ctx: a #GMimeCryptoContext
+ *
+ * Gets the key exchange protocol for the crypto context.
+ *
+ * Returns: the key exchange protocol or %NULL if not supported.
+ **/
+const char *
+g_mime_crypto_context_get_key_exchange_protocol (GMimeCryptoContext *ctx)
+{
+	g_return_val_if_fail (GMIME_IS_CRYPTO_CONTEXT (ctx), NULL);
+	
+	return GMIME_CRYPTO_CONTEXT_GET_CLASS (ctx)->get_key_exchange_protocol (ctx);
+}
+
+
 static int
 crypto_sign (GMimeCryptoContext *ctx, const char *userid, GMimeCryptoHash hash,
 	     GMimeStream *istream, GMimeStream *ostream, GError **err)
@@ -332,7 +413,7 @@ g_mime_crypto_context_encrypt (GMimeCryptoContext *ctx, gboolean sign, const cha
 }
 
 
-static GMimeSignatureValidity *
+static GMimeDecryptionResult *
 crypto_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
 		GMimeStream *ostream, GError **err)
 {
@@ -354,15 +435,16 @@ crypto_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
  * cleartext to the output stream.
  *
  * If the encrypted input stream was also signed, the returned
- * #GMimeSignatureValidity will contain a list of signers, each with a
- * #GMimeSignerStatus (among other details).
+ * #GMimeDecryptionResult will have a non-%NULL #GMimeSignatureValidity which
+ * will contain a list of signers, each with a #GMimeSignerStatus (among other
+ * details about each signer).
  *
- * If the encrypted input text was not signed, then the
- * #GMimeSignatureValidity will not contain any signers.
+ * On success, the returned #GMimeDecryptionResult will contain a list of
+ * recipient keys that the original encrypted stream was encrypted to.
  *
- * Returns: a #GMimeSignatureValidity on success or %NULL on error.
+ * Returns: a #GMimeDecryptionResult on success or %NULL on error.
  **/
-GMimeSignatureValidity *
+GMimeDecryptionResult *
 g_mime_crypto_context_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
 			       GMimeStream *ostream, GError **err)
 {
@@ -479,7 +561,7 @@ g_mime_signer_new (GMimeSignerStatus status)
 
 /**
  * g_mime_signer_free:
- * @signer: signer
+ * @signer: a #GMimeSigner
  *
  * Frees the singleton signer. Should NOT be used to free signers
  * returned from g_mime_signature_validity_get_signers().
@@ -506,8 +588,8 @@ g_mime_signer_free (GMimeSigner *signer)
  *
  * Returns: the next #GMimeSigner or %NULL when complete.
  **/
-GMimeSigner *
-g_mime_signer_next (GMimeSigner *signer)
+const GMimeSigner *
+g_mime_signer_next (const GMimeSigner *signer)
 {
 	g_return_val_if_fail (signer != NULL, NULL);
 	
@@ -616,6 +698,72 @@ g_mime_signer_get_trust (const GMimeSigner *signer)
 }
 
 
+/**
+ * g_mime_signer_set_sig_class:
+ * @signer: a #GMimeSigner
+ * @sig_class: signature class
+ *
+ * Set the signer's signature class.
+ **/
+void
+g_mime_signer_set_sig_class (GMimeSigner *signer, int sig_class)
+{
+	g_return_if_fail (signer != NULL);
+	
+	signer->sig_class = (unsigned int) (sig_class & 0xff);
+}
+
+
+/**
+ * g_mime_signer_get_sig_class:
+ * @signer: a #GMimeSigner
+ *
+ * Get the signer's signature class.
+ *
+ * Returns: the signer's signature class.
+ **/
+int
+g_mime_signer_get_sig_class (const GMimeSigner *signer)
+{
+	g_return_val_if_fail (signer != NULL, 0);
+	
+	return signer->sig_class;
+}
+
+
+/**
+ * g_mime_signer_set_sig_version:
+ * @signer: a #GMimeSigner
+ * @version: signature version
+ *
+ * Set the signer's signature version.
+ **/
+void
+g_mime_signer_set_sig_version (GMimeSigner *signer, int version)
+{
+	g_return_if_fail (signer != NULL);
+	
+	signer->sig_ver = (unsigned int) (version & 0xff);
+}
+
+
+/**
+ * g_mime_signer_get_sig_version:
+ * @signer: a #GMimeSigner
+ *
+ * Get the signer's signature version.
+ *
+ * Returns: the signer's signature version.
+ **/
+int
+g_mime_signer_get_sig_version (const GMimeSigner *signer)
+{
+	g_return_val_if_fail (signer != NULL, 0);
+	
+	return signer->sig_ver;
+}
+
+
 /**
  * g_mime_signer_set_pubkey_algo:
  * @signer: a #GMimeSigner
@@ -1042,7 +1190,7 @@ g_mime_signature_validity_new (void)
 
 /**
  * g_mime_signature_validity_free:
- * @validity: signature validity
+ * @validity: a #GMimeSignatureValidity
  *
  * Frees the memory used by @validity back to the system.
  **/
@@ -1069,7 +1217,7 @@ g_mime_signature_validity_free (GMimeSignatureValidity *validity)
 
 /**
  * g_mime_signature_validity_get_details:
- * @validity: signature validity
+ * @validity: a #GMimeSignatureValidity
  *
  * Gets any user-readable status details.
  *
@@ -1086,7 +1234,7 @@ g_mime_signature_validity_get_details (const GMimeSignatureValidity *validity)
 
 /**
  * g_mime_signature_validity_set_details:
- * @validity: signature validity
+ * @validity: a #GMimeSignatureValidity
  * @details: details string
  *
  * Sets @details as the status details string on @validity.
@@ -1103,7 +1251,7 @@ g_mime_signature_validity_set_details (GMimeSignatureValidity *validity, const c
 
 /**
  * g_mime_signature_validity_get_signers:
- * @validity: signature validity
+ * @validity: a #GMimeSignatureValidity
  *
  * Gets the list of signers.
  *
@@ -1122,8 +1270,8 @@ g_mime_signature_validity_get_signers (const GMimeSignatureValidity *validity)
 
 /**
  * g_mime_signature_validity_add_signer:
- * @validity: signature validity
- * @signer: signer
+ * @validity: a #GMimeSignatureValidity
+ * @signer: a #GMimeSigner
  *
  * Adds @signer to the list of signers on @validity. Once the signer
  * is added, it must NOT be freed.
@@ -1146,3 +1294,322 @@ g_mime_signature_validity_add_signer (GMimeSignatureValidity *validity, GMimeSig
 		s->next = signer;
 	}
 }
+
+
+/**
+ * g_mime_crypto_recipient_new:
+ *
+ * Allocates an new #GMimeCryptoRecipient. This function is meant to be used
+ * in #GMimeCryptoContext subclasses when allocating recipients to add to a
+ * #GMimeDecryptionResult.
+ *
+ * Returns: a new #GMimeCryptoRecipient.
+ **/
+GMimeCryptoRecipient *
+g_mime_crypto_recipient_new (void)
+{
+	GMimeCryptoRecipient *recipient;
+	
+	recipient = g_slice_new (GMimeCryptoRecipient);
+	recipient->pubkey_algo = GMIME_CRYPTO_PUBKEY_ALGO_DEFAULT;
+	recipient->keyid = NULL;
+	recipient->next = NULL;
+	
+	return recipient;
+}
+
+
+/**
+ * g_mime_crypto_recipient_free:
+ * @recipient: a #GMimeCryptoRecipient
+ *
+ * Frees the singleton recipient. Should NOT be used to free recipients
+ * returned from g_mime_signature_validity_get_recipients().
+ **/
+void
+g_mime_crypto_recipient_free (GMimeCryptoRecipient *recipient)
+{
+	g_free (recipient->keyid);
+	
+	g_slice_free (GMimeCryptoRecipient, recipient);
+}
+
+
+/**
+ * g_mime_crypto_recipient_next:
+ * @recipient: a #GMimeCryptoRecipient
+ *
+ * Advance to the next recipient.
+ *
+ * Returns: the next #GMimeCryptoRecipient or %NULL when complete.
+ **/
+const GMimeCryptoRecipient *
+g_mime_crypto_recipient_next (const GMimeCryptoRecipient *recipient)
+{
+	g_return_val_if_fail (recipient != NULL, NULL);
+	
+	return recipient->next;
+}
+
+
+/**
+ * g_mime_crypto_recipient_set_pubkey_algo:
+ * @recipient: a #GMimeCryptoRecipient
+ * @pubkey_algo: a #GMimeCryptoPubKeyAlgo
+ *
+ * Set the public-key algorithm used by the recipient.
+ **/
+void
+g_mime_crypto_recipient_set_pubkey_algo (GMimeCryptoRecipient *recipient, GMimeCryptoPubKeyAlgo pubkey_algo)
+{
+	g_return_if_fail (recipient != NULL);
+	
+	recipient->pubkey_algo = pubkey_algo;
+}
+
+
+/**
+ * g_mime_crypto_recipient_get_pubkey_algo:
+ * @recipient: a #GMimeCryptoRecipient
+ *
+ * Get the public-key algorithm used by the recipient.
+ *
+ * Returns: the public-key algorithm used by the recipient.
+ **/
+GMimeCryptoPubKeyAlgo
+g_mime_crypto_recipient_get_pubkey_algo (const GMimeCryptoRecipient *recipient)
+{
+	g_return_val_if_fail (recipient != NULL, GMIME_CRYPTO_PUBKEY_ALGO_DEFAULT);
+	
+	return recipient->pubkey_algo;
+}
+
+
+/**
+ * g_mime_crypto_recipient_set_key_id:
+ * @recipient: a #GMimeCryptoRecipient
+ * @key_id: key id
+ *
+ * Set the recipient's key id.
+ **/
+void
+g_mime_crypto_recipient_set_key_id (GMimeCryptoRecipient *recipient, const char *key_id)
+{
+	g_return_if_fail (recipient != NULL);
+	
+	g_free (recipient->keyid);
+	recipient->keyid = g_strdup (key_id);
+}
+
+
+/**
+ * g_mime_crypto_recipient_get_key_id:
+ * @recipient: a #GMimeCryptoRecipient
+ *
+ * Get the recipient's key id.
+ *
+ * Returns: the recipient's key id.
+ **/
+const char *
+g_mime_crypto_recipient_get_key_id (const GMimeCryptoRecipient *recipient)
+{
+	g_return_val_if_fail (recipient != NULL, NULL);
+	
+	return recipient->keyid;
+}
+
+
+/**
+ * g_mime_decryption_result_new:
+ *
+ * Creates a new #GMimeDecryptionResult.
+ *
+ * Returns: a new #GMimeDecryptionResult.
+ **/
+GMimeDecryptionResult *
+g_mime_decryption_result_new (void)
+{
+	GMimeDecryptionResult *result;
+	
+	result = g_slice_new (GMimeDecryptionResult);
+	result->cipher = GMIME_CRYPTO_CIPHER_ALGO_DEFAULT;
+	result->mdc = GMIME_CRYPTO_HASH_DEFAULT;
+	result->recipients = NULL;
+	result->validity = NULL;
+	
+	return result;
+}
+
+
+/**
+ * g_mime_decryption_result_free:
+ * @result: a #GMimeDecryptionResult
+ *
+ * Frees the memory used by @result back to the system.
+ **/
+void
+g_mime_decryption_result_free (GMimeDecryptionResult *result)
+{
+	GMimeCryptoRecipient *recipient, *next;
+	
+	if (result == NULL)
+		return;
+	
+	recipient = result->recipients;
+	while (recipient != NULL) {
+		next = recipient->next;
+		g_mime_crypto_recipient_free (recipient);
+		recipient = next;
+	}
+	
+	g_mime_signature_validity_free (result->validity);
+	
+	g_slice_free (GMimeDecryptionResult, result);
+}
+
+
+/**
+ * g_mime_decryption_result_get_validity:
+ * @result: a #GMimeDecryptionResult
+ *
+ * Gets the signature validity if the decrypted stream was also signed.
+ *
+ * Returns: a #GMimeSignatureValidity or %NULL if the stream was not signed.
+ **/
+const GMimeSignatureValidity *
+g_mime_decryption_result_get_validity (const GMimeDecryptionResult *result)
+{
+	g_return_val_if_fail (result != NULL, NULL);
+	
+	return result->validity;
+}
+
+
+/**
+ * g_mime_decryption_result_set_validity:
+ * @result: a #GMimeDecryptionResult
+ * @validity: a #GMimeSignatureValidity
+ *
+ * Sets @validity as the #GMimeDecryptionResult.
+ **/
+void
+g_mime_decryption_result_set_validity (GMimeDecryptionResult *result, GMimeSignatureValidity *validity)
+{
+	g_return_if_fail (result != NULL);
+	
+	g_mime_signature_validity_free (result->validity);
+	result->validity = validity;
+}
+
+
+/**
+ * g_mime_decryption_result_get_recipients:
+ * @result: signature result
+ *
+ * Gets the list of recipients.
+ *
+ * Returns: a #GMimeCryptoRecipient list which contain further information such
+ * as trust and crypto keys. These recipients are part of the
+ * #GMimeDecryptionResult and should NOT be freed individually.
+ **/
+const GMimeCryptoRecipient *
+g_mime_decryption_result_get_recipients (const GMimeDecryptionResult *result)
+{
+	g_return_val_if_fail (result != NULL, NULL);
+	
+	return result->recipients;
+}
+
+
+/**
+ * g_mime_decryption_result_add_recipient:
+ * @result: a #GMimeDecryptionResult
+ * @recipient: a #GMimeCryptoRecipient
+ *
+ * Adds @recipient to the list of recipients on @result. Once the recipient
+ * is added, it must NOT be freed.
+ **/
+void
+g_mime_decryption_result_add_recipient (GMimeDecryptionResult *result, GMimeCryptoRecipient *recipient)
+{
+	GMimeCryptoRecipient *r;
+	
+	g_return_if_fail (result != NULL);
+	g_return_if_fail (recipient != NULL);
+	
+	if (result->recipients == NULL) {
+		result->recipients = recipient;
+	} else {
+		r = result->recipients;
+		while (r->next != NULL)
+			r = r->next;
+		
+		r->next = recipient;
+	}
+}
+
+
+/**
+ * g_mime_decryption_result_set_cipher:
+ * @result: a #GMimeDecryptionResult
+ * @cipher: a #GMimeCryptoCipherAlgo
+ *
+ * Set the cipher algorithm used.
+ **/
+void
+g_mime_decryption_result_set_cipher (GMimeDecryptionResult *result, GMimeCryptoCipherAlgo cipher)
+{
+	g_return_if_fail (result != NULL);
+	
+	result->cipher = cipher;
+}
+
+
+/**
+ * g_mime_decryption_result_get_cipher:
+ * @result: a #GMimeDecryptionResult
+ *
+ * Get the cipher algorithm used.
+ *
+ * Returns: the cipher algorithm used.
+ **/
+GMimeCryptoCipherAlgo
+g_mime_decryption_result_get_cipher (const GMimeDecryptionResult *result)
+{
+	g_return_val_if_fail (result != NULL, GMIME_CRYPTO_CIPHER_ALGO_DEFAULT);
+	
+	return result->cipher;
+}
+
+
+/**
+ * g_mime_decryption_result_set_mdc:
+ * @result: a #GMimeDecryptionResult
+ * @mdc: a #GMimeCryptoHash
+ *
+ * Set the mdc digest algorithm used.
+ **/
+void
+g_mime_decryption_result_set_mdc (GMimeDecryptionResult *result, GMimeCryptoHash mdc)
+{
+	g_return_if_fail (result != NULL);
+	
+	result->mdc = mdc;
+}
+
+
+/**
+ * g_mime_decryption_result_get_mdc:
+ * @result: a #GMimeDecryptionResult
+ *
+ * Get the mdc digest algorithm used.
+ *
+ * Returns: the mdc digest algorithm used.
+ **/
+GMimeCryptoHash
+g_mime_decryption_result_get_mdc (const GMimeDecryptionResult *result)
+{
+	g_return_val_if_fail (result != NULL, GMIME_CRYPTO_HASH_DEFAULT);
+	
+	return result->mdc;
+}
