@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2011 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -76,6 +76,12 @@ static GMimeCryptoHash pkcs7_hash_id (GMimeCryptoContext *ctx, const char *hash)
 
 static const char *pkcs7_hash_name (GMimeCryptoContext *ctx, GMimeCryptoHash hash);
 
+static const char *pkcs7_get_signature_protocol (GMimeCryptoContext *ctx);
+
+static const char *pkcs7_get_encryption_protocol (GMimeCryptoContext *ctx);
+
+static const char *pkcs7_get_key_exchange_protocol (GMimeCryptoContext *ctx);
+
 static int pkcs7_sign (GMimeCryptoContext *ctx, const char *userid,
 		       GMimeCryptoHash hash, GMimeStream *istream,
 		       GMimeStream *ostream, GError **err);
@@ -88,8 +94,8 @@ static int pkcs7_encrypt (GMimeCryptoContext *ctx, gboolean sign, const char *us
 			  GMimeCryptoHash hash, GPtrArray *recipients, GMimeStream *istream,
 			  GMimeStream *ostream, GError **err);
 
-static GMimeSignatureValidity *pkcs7_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
-					      GMimeStream *ostream, GError **err);
+static GMimeDecryptionResult *pkcs7_decrypt (GMimeCryptoContext *ctx, GMimeStream *istream,
+					     GMimeStream *ostream, GError **err);
 
 static int pkcs7_import_keys (GMimeCryptoContext *ctx, GMimeStream *istream,
 			      GError **err);
@@ -144,6 +150,9 @@ g_mime_pkcs7_context_class_init (GMimePkcs7ContextClass *klass)
 	crypto_class->decrypt = pkcs7_decrypt;
 	crypto_class->import_keys = pkcs7_import_keys;
 	crypto_class->export_keys = pkcs7_export_keys;
+	crypto_class->get_signature_protocol = pkcs7_get_signature_protocol;
+	crypto_class->get_encryption_protocol = pkcs7_get_encryption_protocol;
+	crypto_class->get_key_exchange_protocol = pkcs7_get_key_exchange_protocol;
 }
 
 static void
@@ -156,10 +165,6 @@ g_mime_pkcs7_context_init (GMimePkcs7Context *ctx, GMimePkcs7ContextClass *klass
 #ifdef ENABLE_SMIME
 	ctx->priv->ctx = NULL;
 #endif
-	
-	crypto->sign_protocol = "application/pkcs7-signature";
-	crypto->encrypt_protocol = "application/pkcs7-mime";
-	crypto->key_protocol = "application/pkcs7-keys";
 }
 
 static void
@@ -240,6 +245,24 @@ pkcs7_hash_name (GMimeCryptoContext *ctx, GMimeCryptoHash hash)
 	}
 }
 
+static const char *
+pkcs7_get_signature_protocol (GMimeCryptoContext *ctx)
+{
+	return "application/pkcs7-signature";
+}
+
+static const char *
+pkcs7_get_encryption_protocol (GMimeCryptoContext *ctx)
+{
+	return "application/pkcs7-mime";
+}
+
+static const char *
+pkcs7_get_key_exchange_protocol (GMimeCryptoContext *ctx)
+{
+	return "application/pkcs7-keys";
+}
+
 #ifdef ENABLE_SMIME
 static gpgme_error_t
 pkcs7_passphrase_cb (void *hook, const char *uid_hint, const char *passphrase_info, int prev_was_bad, int fd)
@@ -280,6 +303,19 @@ pkcs7_stream_write (void *stream, const void *buffer, size_t size)
 	return g_mime_stream_write ((GMimeStream *) stream, (const char *) buffer, size);
 }
 
+static off_t
+pkcs7_stream_seek (void *stream, off_t offset, int whence)
+{
+	switch (whence) {
+	case SEEK_SET:
+		return (off_t) g_mime_stream_seek ((GMimeStream *) stream, (gint64) offset, GMIME_STREAM_SEEK_SET);
+	case SEEK_CUR:
+		return (off_t) g_mime_stream_seek ((GMimeStream *) stream, (gint64) offset, GMIME_STREAM_SEEK_CUR);
+	case SEEK_END:
+		return (off_t) g_mime_stream_seek ((GMimeStream *) stream, (gint64) offset, GMIME_STREAM_SEEK_END);
+	}
+}
+
 static void
 pkcs7_stream_free (void *stream)
 {
@@ -289,7 +325,7 @@ pkcs7_stream_free (void *stream)
 static struct gpgme_data_cbs pkcs7_stream_funcs = {
 	pkcs7_stream_read,
 	pkcs7_stream_write,
-	NULL,
+	pkcs7_stream_seek,
 	pkcs7_stream_free
 };
 
@@ -467,39 +503,6 @@ pkcs7_trust (gpgme_validity_t trust)
 	}
 }
 
-static GMimeCryptoPubKeyAlgo
-pkcs7_pubkey_algo (id)
-{
-	switch (id) {
-	case GPGME_PK_RSA: return GMIME_CRYPTO_PUBKEY_ALGO_RSA;
-	case GPGME_PK_RSA_E: return GMIME_CRYPTO_PUBKEY_ALGO_RSA_E;
-	case GPGME_PK_RSA_S: return GMIME_CRYPTO_PUBKEY_ALGO_RSA_S;
-	case GPGME_PK_ELG_E: return GMIME_CRYPTO_PUBKEY_ALGO_ELG_E;
-	case GPGME_PK_DSA: return GMIME_CRYPTO_PUBKEY_ALGO_DSA;
-	case GPGME_PK_ELG: return GMIME_CRYPTO_PUBKEY_ALGO_ELG;
-	default: return GMIME_CRYPTO_PUBKEY_ALGO_DEFAULT;
-	}
-}
-
-static GMimeCryptoHash
-pkcs7_hash_algo (gpgme_hash_algo_t id)
-{
-	switch (id) {
-	case GPGME_MD_NONE: return GMIME_CRYPTO_HASH_DEFAULT;
-	case GPGME_MD_MD5: return GMIME_CRYPTO_HASH_MD5;
-	case GPGME_MD_SHA1: return GMIME_CRYPTO_HASH_SHA1;
-	case GPGME_MD_RMD160: return GMIME_CRYPTO_HASH_RIPEMD160;
-	case GPGME_MD_MD2: return GMIME_CRYPTO_HASH_MD2;
-	case GPGME_MD_TIGER: return GMIME_CRYPTO_HASH_TIGER192;
-	case GPGME_MD_HAVAL: return GMIME_CRYPTO_HASH_HAVAL5160;
-	case GPGME_MD_SHA256: return GMIME_CRYPTO_HASH_SHA256;
-	case GPGME_MD_SHA384: return GMIME_CRYPTO_HASH_SHA384;
-	case GPGME_MD_SHA512: return GMIME_CRYPTO_HASH_SHA512;
-	case GPGME_MD_MD4: return GMIME_CRYPTO_HASH_MD4;
-	default: return GMIME_CRYPTO_HASH_DEFAULT;
-	}
-}
-
 static GMimeSignatureValidity *
 pkcs7_get_validity (Pkcs7Ctx *pkcs7, gboolean verify)
 {
@@ -511,12 +514,12 @@ pkcs7_get_validity (Pkcs7Ctx *pkcs7, gboolean verify)
 	gpgme_user_id_t uid;
 	gpgme_key_t key;
 	
-	/* create a new signature validity to return */
-	validity = g_mime_signature_validity_new ();
-	
 	/* get the signature verification results from GpgMe */
 	if (!(result = gpgme_op_verify_result (pkcs7->ctx)) || !result->signatures)
-		return validity;
+		return verify ? g_mime_signature_validity_new () : NULL;
+	
+	/* create a new signature validity to return */
+	validity = g_mime_signature_validity_new ();
 	
 	/* collect the signers for this signature */
 	signers = (GMimeSigner *) &validity->signers;
@@ -530,8 +533,8 @@ pkcs7_get_validity (Pkcs7Ctx *pkcs7, gboolean verify)
 		signers->next = signer;
 		signers = signer;
 		
-		g_mime_signer_set_pubkey_algo (signer, pkcs7_pubkey_algo (sig->pubkey_algo));
-		g_mime_signer_set_hash_algo (signer, pkcs7_hash_algo (sig->hash_algo));
+		g_mime_signer_set_pubkey_algo (signer, sig->pubkey_algo);
+		g_mime_signer_set_hash_algo (signer, sig->hash_algo);
 		g_mime_signer_set_sig_expires (signer, sig->exp_timestamp);
 		g_mime_signer_set_sig_created (signer, sig->timestamp);
 		g_mime_signer_set_fingerprint (signer, sig->fpr);
@@ -742,13 +745,46 @@ pkcs7_encrypt (GMimeCryptoContext *context, gboolean sign, const char *userid,
 }
 
 
-static GMimeSignatureValidity *
+static GMimeDecryptionResult *
+pkcs7_get_decrypt_result (Pkcs7Ctx *pkcs7)
+{
+	GMimeCryptoRecipient *recipient, *recipients;
+	GMimeDecryptionResult *result;
+	gpgme_decrypt_result_t res;
+	gpgme_recipient_t recip;
+	
+	result = g_mime_decryption_result_new ();
+	result->validity = pkcs7_get_validity (pkcs7, FALSE);
+	
+	if (!(res = gpgme_op_decrypt_result (pkcs7->ctx)) || !res->recipients)
+		return result;
+	
+	recipients = (GMimeCryptoRecipient *) &result->recipients;
+	
+	recip = res->recipients;
+	while (recip != NULL) {
+		recipient = g_mime_crypto_recipient_new ();
+		recipients->next = recipient;
+		recipients = recipient;
+		
+		g_mime_crypto_recipient_set_pubkey_algo (recipient, recip->pubkey_algo);
+		g_mime_crypto_recipient_set_key_id (recipient, recip->keyid);
+		
+		recip = recip->next;
+	}
+	
+	return result;
+}
+
+static GMimeDecryptionResult *
 pkcs7_decrypt (GMimeCryptoContext *context, GMimeStream *istream,
 	       GMimeStream *ostream, GError **err)
 {
 #ifdef ENABLE_SMIME
 	GMimePkcs7Context *ctx = (GMimePkcs7Context *) context;
+	GMimeDecryptionResult *result;
 	Pkcs7Ctx *pkcs7 = ctx->priv;
+	gpgme_decrypt_result_t res;
 	gpgme_data_t input, output;
 	gpgme_error_t error;
 	
@@ -774,8 +810,7 @@ pkcs7_decrypt (GMimeCryptoContext *context, GMimeStream *istream,
 	gpgme_data_release (output);
 	gpgme_data_release (input);
 	
-	/* get/return the pkcs7 signature validity */
-	return pkcs7_get_validity (pkcs7, FALSE);
+	return pkcs7_get_decrypt_result (pkcs7);
 #else
 	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED, _("S/MIME support is not enabled in this build"));
 	
@@ -799,6 +834,7 @@ pkcs7_import_keys (GMimeCryptoContext *context, GMimeStream *istream, GError **e
 	
 	/* import the key(s) */
 	if ((error = gpgme_op_import (pkcs7->ctx, keydata)) != GPG_ERR_NO_ERROR) {
+		//printf ("import error (%d): %s\n", error & GPG_ERR_CODE_MASK, gpg_strerror (error));
 		g_set_error (err, GMIME_GPGME_ERROR, error, _("Could not import key data"));
 		gpgme_data_release (keydata);
 		return -1;
