@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2010 Jeffrey Stedfast
+ *  Copyright (C) 2000-2011 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -260,9 +260,9 @@ test_multipart_signed (GMimeCryptoContext *ctx)
 static void
 test_multipart_encrypted (GMimeCryptoContext *ctx, gboolean sign)
 {
-	const GMimeSignatureValidity *sv;
 	GMimeStream *cleartext, *stream;
 	GMimeMultipartEncrypted *mpe;
+	GMimeDecryptionResult *result;
 	GMimeDataWrapper *content;
 	GMimeObject *decrypted;
 	GPtrArray *recipients;
@@ -342,25 +342,28 @@ test_multipart_encrypted (GMimeCryptoContext *ctx, gboolean sign)
 	mpe = (GMimeMultipartEncrypted *) message->mime_part;
 	
 	/* okay, now to test our decrypt function... */
-	decrypted = g_mime_multipart_encrypted_decrypt (mpe, ctx, &err);
+	decrypted = g_mime_multipart_encrypted_decrypt (mpe, ctx, &result, &err);
 	if (!decrypted || err != NULL) {
 		ex = exception_new ("decryption failed: %s", err->message);
+		g_mime_decryption_result_free (result);
 		g_object_unref (cleartext);
 		g_error_free (err);
 		throw (ex);
 	}
 	
-	sv = g_mime_multipart_encrypted_get_signature_validity (mpe);
-	v(print_verify_results (sv));
+	if (result->validity)
+		v(print_verify_results (result->validity));
 	
 	if (sign) {
-		if (get_sig_status (sv->signers) != GMIME_SIGNER_STATUS_GOOD)
+		if (!result->validity || get_sig_status (result->validity->signers) != GMIME_SIGNER_STATUS_GOOD)
 			ex = exception_new ("signature status expected to be GOOD");
 	} else {
-		if (sv->signers != NULL)
+		if (result->validity && result->validity->signers != NULL)
 			ex = exception_new ("signature status expected to be NONE");
 	}
 	
+	g_mime_decryption_result_free (result);
+	
 	if (ex != NULL) {
 		g_object_unref (cleartext);
 		throw (ex);
